@@ -34,7 +34,8 @@
 #include <boost/gil/gil_all.hpp>
 #pragma GCC diagnostic pop
 
-// agg
+#pragma GCC diagnostic push
+#include <mapnik/warning_ignore_agg.hpp>
 #include "agg_basics.h"
 #include "agg_rendering_buffer.h"
 #include "agg_color_rgba.h"
@@ -42,6 +43,8 @@
 #include "agg_scanline_u.h"
 #include "agg_blur.h"
 #include "agg_gradient_lut.h"
+#pragma GCC diagnostic pop
+
 // stl
 #include <cmath>
 
@@ -163,7 +166,11 @@ struct double_buffer
 template <typename Src, typename Dst, typename Conv>
 void process_channel_impl (Src const& src, Dst & dst, Conv const& k)
 {
+#if BOOST_VERSION >= 106800
+    using bits32f = boost::gil::float32_t;
+#else
     using boost::gil::bits32f;
+#endif
 
     bits32f out_value =
         k[0]*src[0] + k[1]*src[1] + k[2]*src[2] +
@@ -208,7 +215,11 @@ void process_channel (Src const& src, Dst & dst, mapnik::filter::edge_detect)
 template <typename Src, typename Dst>
 void process_channel (Src const& src, Dst & dst, mapnik::filter::sobel)
 {
+#if BOOST_VERSION >= 106800
+    using bits32f = boost::gil::float32_t;
+#else
     using boost::gil::bits32f;
+#endif
 
     bits32f x_gradient = (src[2] + 2*src[5] + src[8])
         - (src[0] + 2*src[3] + src[6]);
@@ -228,7 +239,11 @@ void process_channel (Src const& src, Dst & dst, mapnik::filter::sobel)
 template <typename Src, typename Dst, typename Filter>
 void apply_convolution_3x3(Src const& src_view, Dst & dst_view, Filter const& filter)
 {
+#if BOOST_VERSION >= 106800
+    using bits32f = boost::gil::float32_t;
+#else
     using boost::gil::bits32f;
+#endif
     using boost::gil::point2;
 
     // p0 p1 p2
@@ -387,7 +402,7 @@ void apply_convolution_3x3(Src const& src_view, Dst & dst_view, Filter const& fi
 }
 
 template <typename Src, typename Filter>
-void apply_filter(Src & src, Filter const& filter)
+void apply_filter(Src & src, Filter const& filter, double /*scale_factor*/)
 {
     demultiply_alpha(src);
     double_buffer<Src> tb(src);
@@ -395,12 +410,12 @@ void apply_filter(Src & src, Filter const& filter)
 }
 
 template <typename Src>
-void apply_filter(Src & src, agg_stack_blur const& op)
+void apply_filter(Src & src, agg_stack_blur const& op, double scale_factor)
 {
     premultiply_alpha(src);
     agg::rendering_buffer buf(src.bytes(),src.width(),src.height(), src.row_size());
     agg::pixfmt_rgba32_pre pixf(buf);
-    agg::stack_blur_rgba32(pixf,op.rx,op.ry);
+    agg::stack_blur_rgba32(pixf, op.rx * scale_factor, op.ry * scale_factor);
 }
 
 inline double channel_delta(double source, double match)
@@ -417,7 +432,7 @@ inline uint8_t apply_alpha_shift(double source, double match, double alpha)
 }
 
 template <typename Src>
-void apply_filter(Src & src, color_to_alpha const& op)
+void apply_filter(Src & src, color_to_alpha const& op, double /*scale_factor*/)
 {
     using namespace boost::gil;
     bool premultiplied = src.get_premultiplied();
@@ -478,7 +493,7 @@ void apply_filter(Src & src, color_to_alpha const& op)
 }
 
 template <typename Src>
-void apply_filter(Src & src, colorize_alpha const& op)
+void apply_filter(Src & src, colorize_alpha const& op, double /*scale_factor*/)
 {
     using namespace boost::gil;
     std::ptrdiff_t size = op.size();
@@ -587,7 +602,7 @@ void apply_filter(Src & src, colorize_alpha const& op)
 }
 
 template <typename Src>
-void apply_filter(Src & src, scale_hsla const& transform)
+void apply_filter(Src & src, scale_hsla const& transform, double /*scale_factor*/)
 {
     using namespace boost::gil;
     bool tinting = !transform.is_identity();
@@ -799,25 +814,25 @@ void color_blind_filter(Src & src, ColorBlindFilter const& op)
 }
 
 template <typename Src>
-void apply_filter(Src & src, color_blind_protanope const& op)
+void apply_filter(Src & src, color_blind_protanope const& op, double /*scale_factor*/)
 {
     color_blind_filter(src, op);
 }
 
 template <typename Src>
-void apply_filter(Src & src, color_blind_deuteranope const& op)
+void apply_filter(Src & src, color_blind_deuteranope const& op, double /*scale_factor*/)
 {
     color_blind_filter(src, op);
 }
 
 template <typename Src>
-void apply_filter(Src & src, color_blind_tritanope const& op)
+void apply_filter(Src & src, color_blind_tritanope const& op, double /*scale_factor*/)
 {
     color_blind_filter(src, op);
 }
 
 template <typename Src>
-void apply_filter(Src & src, gray const& /*op*/)
+void apply_filter(Src & src, gray const& /*op*/, double /*scale_factor*/)
 {
     premultiply_alpha(src);
     using namespace boost::gil;
@@ -868,7 +883,7 @@ void x_gradient_impl(Src const& src_view, Dst const& dst_view)
 }
 
 template <typename Src>
-void apply_filter(Src & src, x_gradient const& /*op*/)
+void apply_filter(Src & src, x_gradient const& /*op*/, double /*scale_factor*/)
 {
     premultiply_alpha(src);
     double_buffer<Src> tb(src);
@@ -876,7 +891,7 @@ void apply_filter(Src & src, x_gradient const& /*op*/)
 }
 
 template <typename Src>
-void apply_filter(Src & src, y_gradient const& /*op*/)
+void apply_filter(Src & src, y_gradient const& /*op*/, double /*scale_factor*/)
 {
     premultiply_alpha(src);
     double_buffer<Src> tb(src);
@@ -885,7 +900,7 @@ void apply_filter(Src & src, y_gradient const& /*op*/)
 }
 
 template <typename Src>
-void apply_filter(Src & src, invert const& /*op*/)
+void apply_filter(Src & src, invert const& /*op*/, double /*scale_factor*/)
 {
     premultiply_alpha(src);
     using namespace boost::gil;
@@ -913,16 +928,18 @@ void apply_filter(Src & src, invert const& /*op*/)
 template <typename Src>
 struct filter_visitor
 {
-    filter_visitor(Src & src)
-    : src_(src) {}
+    filter_visitor(Src & src, double scale_factor=1.0)
+    : src_(src),
+      scale_factor_(scale_factor) {}
 
     template <typename T>
     void operator () (T const& filter) const
     {
-        apply_filter(src_, filter);
+        apply_filter(src_, filter, scale_factor_);
     }
 
     Src & src_;
+    double scale_factor_;
 };
 
 struct filter_radius_visitor
@@ -941,14 +958,14 @@ struct filter_radius_visitor
 };
 
 template<typename Src>
-void filter_image(Src & src, std::string const& filter)
+void filter_image(Src & src, std::string const& filter, double scale_factor=1)
 {
     std::vector<filter_type> filter_vector;
     if(!parse_image_filters(filter, filter_vector))
     {
         throw std::runtime_error("Failed to parse filter argument in filter_image: '" + filter + "'");
     }
-    filter_visitor<Src> visitor(src);
+    filter_visitor<Src> visitor(src, scale_factor);
     for (filter_type const& filter_tag : filter_vector)
     {
         util::apply_visitor(visitor, filter_tag);
@@ -956,7 +973,7 @@ void filter_image(Src & src, std::string const& filter)
 }
 
 template<typename Src>
-Src filter_image(Src const& src, std::string const& filter)
+Src filter_image(Src const& src, std::string const& filter, double scale_factor=1)
 {
     std::vector<filter_type> filter_vector;
     if(!parse_image_filters(filter, filter_vector))
@@ -964,7 +981,7 @@ Src filter_image(Src const& src, std::string const& filter)
         throw std::runtime_error("Failed to parse filter argument in filter_image: '" + filter + "'");
     }
     Src new_src(src);
-    filter_visitor<Src> visitor(new_src);
+    filter_visitor<Src> visitor(new_src, scale_factor);
     for (filter_type const& filter_tag : filter_vector)
     {
         util::apply_visitor(visitor, filter_tag);
